import java.util.*;

public class Car
{
  int id,capacity;
  int pos,vel,acc;
  Global global;
  Vector passengers;
  int goal; // 0..nfloors-1 (-1=undefined)
  int dir; // -1=down, +1=up, 0=neither
  boolean open; 

  public Car(Global g,int i) 
  { 
    global = g; 
    id = i;
    capacity = 5;
    passengers = new Vector(); 
    pos = vel = acc = dir = 0; 
    goal = -1;
    open = false;
  }

  public void print() 
  { 
    int target=goal*10;
    if (goal==-1) target = -1;
    //System.out.format("elevator: id=%d dir=%d target=%d pos=%d vel=%d acc=%d open=%s load=%d: ",id,dir,target,pos,vel,acc,open,passengers.size()); 
    System.out.format("elevator: id=%d target=%d pos=%d vel=%d acc=%d open=%s dir=%d: ",id,target,pos,vel,acc,open,dir); 
    for (int i=0 ; i<passengers.size() ; i++) { Passenger p=(Passenger)passengers.elementAt(i); System.out.format("id=%d/dest=%d/wait=%d ",p.id,p.dest,p.wait_time()); }
    System.out.println();
  }

  int abs(int x) { if (x<0) return -x; else return x; }

  public void go_to(int fl) { goal = fl; }

  Boolean at_goal() { if (goal>=0 && pos==goal*10 && vel==0 && acc==0) return true; else return false; }

  int get_floor() { if (pos%10!=0) return -1; return (int)(pos/10); } // returns -1 if between floors

  public void set_accel(int x)
  {
    if (x>0) acc = 1;
    else if (x<0) acc = -1;
    else acc = 0;
  }

  public void time_step()
  {
    if (open==true) exchange_passengers(); // door automatically closes when done
    else if (at_goal()==true) open_door();
    else if (goal==-1) decide();
    else
    {
      // do update after show current state, but before adjusting accel
      update_dynamics(); 
      adjust_accel(); 
    }
  }

  void exchange_passengers()
  {
    int fl=get_floor();
    Floor floor=global.get_floor(fl);

    if (open==true)
    {
      for (int i=0 ; i<passengers.size() ; i++)
      {
        Passenger p=((Passenger)passengers.elementAt(i));
        if (p.dest==fl) { discharge(p); return; }
      }
      for (int i=0 ; i<floor.queue.size() ; i++)
      {
        Passenger p=((Passenger)floor.queue.elementAt(i));
        if (passengers.size()<capacity && p.dir()==dir) { load(p,floor); return; }
      }
      close_door();
    }
  }

  void decide()
  {
    // set goal
  }
  
  void adjust_accel()
  {
    if (goal>0)
    {
      int futurepos=pos+vel;
      int targetpos=goal*10,targetvel=0;
      if (targetpos>futurepos+10) targetvel = 4;
      else if (targetpos>=futurepos+6) targetvel = 3;
      else if (targetpos>=futurepos+3) targetvel = 2;
      else if (targetpos>=futurepos+1) targetvel = 1;
      else if (targetpos==futurepos) targetvel = 0;
      else if (targetpos>futurepos-3) targetvel = -1;
      else if (targetpos>futurepos-6) targetvel = -2;
      else if (targetpos>futurepos-10) targetvel = -3;
      else targetvel = -4;

      if (targetvel>vel) set_accel(+1); 
      else if (targetvel<vel) set_accel(-1); 
      else set_accel(0);
    }
  }
 
  void update_dynamics()
  {
    // update pos first, given current vel (before update by acc)
    pos += vel; 
    if (pos<0) pos = 0;
    int n=global.floors.size()-1;
    if (pos>10*n) pos = 10*n;

    if (acc==1) vel++;
    if (acc==-1) vel--;
    if (vel>4) vel = 4; // max velocity
    if (vel<-4) vel = -4;
  }

  void discharge(Passenger p)
  {
System.out.format("car %d discharging passenger at floor %d\n",id,get_floor());
    passengers.remove(p);
    p.arrived();
  }

  void load(Passenger p,Floor f)
  {
System.out.format("car %d loading passenger at floor %d\n",id,get_floor());
    passengers.add(p);
    f.dequeue(p);
  }

  public void open_door()
  {
    if (vel!=0 || pos%10!=0) global.error("can't open door - car not at floor");
    open = true;
System.out.format("car %d opening doors on floor %d\n",id,get_floor());
    int fl=get_floor();
    Floor floor=global.get_floor(fl);
    if (fl==0) dir = +1; // special case: if ground floor, always switch to going up
    if (fl==global.floors.size()-1) dir = -1;
    if (dir==0)
    {
      dir = 1; // if up is pressed, or neither (default)
      if (floor.pressed(-1)==1) dir = -1;
    }
    floor.reset_button(dir); 
    goal = -1;
  }  

  public void close_door()
  {
    open = false;
    global.get_floor(get_floor()).check_buttons(); // in case some remain in queue
  }  
}
